/*

    this source file is a part of 
    lp_parse - a preprocessor for MIP models
    Copyright (C) 1996  Ari Jolma

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    for more information please contact 
    Ari Jolma
    Helsinki University of Technology
    Laboratory of Water Resources Engineering
    Building W
    02150 Espoo, Finland
    E-mail: Ari.Jolma@hut.fi

*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "util.h"
#include "hash.h"

int init_item(ITEM *item, char *name) {
  item->next = NULL;
  if (name) 
    try(item->name = STRDUP(name))
  else 
    item->name = NULL;
  return TRUE;
}

ITEM *new_item(char *name) {
  ITEM *item;
  try(item = (ITEM *)MALLOC(sizeof(ITEM)));
  ctry(init_item(item, name));
  return item;
catch:
  dispose_item(item);
  return NULL;
}

void delete_item(ITEM *item) {
  if (item) {
    if (item->name) free (item->name);
  }
}

void dispose_item(ITEM *item) {
  if (item) {
    delete_item(item);
    free (item);
  }
}

#define HASHSIZE 256

int hashfct(const char *name ) {
  int h = 0;
  while (*name) {
    h += (int)*name;
    name++;
  }
  return h;
}  

HASH *new_hash() {
  HASH *hash;
  int i;
  try(hash = CALLOC(HASHSIZE, sizeof(ITEM *)));
  for (i=0;i<HASHSIZE;i++) hash[i] = NULL;
  return hash;
}

void delete_hash(HASH *hash) {
  if(hash) free(hash);
}

void dispose_hash(HASH *hash) {
  ITEM *a,*b;
  int i;
  for (i=0;i<HASHSIZE;i++) {
    if (hash[i]) {
      a = hash[i];
      do {	
	b = a->next;
	dispose_item(a);
	a = b;
      } while (a);
    }
  }
  delete_hash(hash);
}

void ahash(HASH *hash, ITEM *item) {
  ITEM **h = &hash[hashfct(item->name) % HASHSIZE];
  while (*h) h = (ITEM **)&((*h)->next);
  *h = item;
}

ITEM *ghash(HASH *hash, char *name) {
  ITEM *h = hash[hashfct(name) % HASHSIZE];
  while (h) {
    if (strcmp(h->name, name) == 0) return h;
    h = h->next;
  }
  return NULL;
}

int nitems(HASH *hash) {
  int i,k=0;
  ITEM *h;
  for (i=0;i<HASHSIZE;i++) {
    if ((h = hash[i])) {
      do {
	k++;
	h = h->next;
      } while (h);
    }
  }
  return k;
}

ITEM **items(HASH *hash, int nitems) {
  int i,j=0;
  ITEM *h;
  ITEM **_items = CALLOC(nitems, sizeof(ITEM *));
  for (i=0;i<HASHSIZE;i++) {
    if ((h = hash[i])) {
      do {
	_items[j] = h;
	j++;
	h = h->next;
      } while (h);
    }
  }
  return _items;
}

